using HugsLib;
using HugsLib.Utils;
using System;
using System.Collections.Generic;
using System.Linq;
using Verse;
using RimWorld;

namespace rjw
{
	public class SaveStorage : ModBase
	{
		public override string ModIdentifier => "RJW";

		public override Version GetVersion()
		{
			//--Log.Message("GetVersion() called");
			return base.GetVersion();
		}

		public static DataStore DataStore;//reference to savegame data, hopefully
		public override void WorldLoaded()
		{
			DataStore = UtilityWorldObjectManager.GetUtilityWorldObject<DataStore>();
		}
		protected override bool HarmonyAutoPatch { get => false; }//first.cs creates harmony and does some convoulted stuff with it
	}

	/// <summary>
	/// Utility rjw data object and a collection of extension methods for Pawn
	/// </summary>
	public class PawnData : IExposable
	{
		//Should probably mix but not shake it with RJW designation classes
		public bool Comfort = false;
		public bool Service = false;
		public bool Breeding = false;
		public bool Milking = false;
		public bool Hero = false;
		public bool BreedingAnimal = false;
		public Pawn Pawn = null;

		public PawnData() { }

		public PawnData(Pawn pawn)
		{
			//Log.Message("Creating pawndata for " + pawn);
			Pawn = pawn;
			//Log.Message("This data is valid " + this.IsValid);
		}

		public void ExposeData()
		{
			Scribe_Values.Look<bool>(ref Comfort, "Comfort", false, true);
			Scribe_Values.Look<bool>(ref Service, "Service", false, true);
			Scribe_Values.Look<bool>(ref Breeding, "Breeding", false, true);
			Scribe_Values.Look<bool>(ref Milking, "Milking", false, true);
			Scribe_Values.Look<bool>(ref Hero, "Hero", false, true);
			Scribe_Values.Look<bool>(ref BreedingAnimal, "BreedingAnimal", false, true);
			Scribe_References.Look<Pawn>(ref this.Pawn, "Pawn");
		}

		public bool IsValid { get { return Pawn != null; } }
	}
	//Shitcode ahead
	public static class PawnDesignations
	{
		public static bool IsDesignatedComfort(this Pawn pawn)
		{
			//return comfort_prisoners.is_designated(pawn);
			if (!CanDesignateComfort(pawn))
				UnDesignateComfort(pawn);

			return SaveStorage.DataStore.GetPawnData(pawn).Comfort;
		}
		public static bool IsDesignatedService(this Pawn pawn)
		{
			if (!CanDesignateService(pawn))
				UnDesignateService(pawn);

			return SaveStorage.DataStore.GetPawnData(pawn).Service;
		}
		public static bool IsDesignatedBreeding(this Pawn pawn)
		{
			if (!CanDesignateBreed(pawn))
				UnDesignateBreeding(pawn);

			return SaveStorage.DataStore.GetPawnData(pawn).Breeding;
		}
		public static bool IsDesignatedMilking(this Pawn pawn)
		{
			if (!CanDesignateMilk(pawn))
				UnDesignateMilking(pawn);

			return SaveStorage.DataStore.GetPawnData(pawn).Milking;
		}
		public static bool IsDesignatedHero(this Pawn pawn)
		{
			return SaveStorage.DataStore.GetPawnData(pawn).Hero;
		}
		public static bool IsDesignatedBreedingAnimal(this Pawn pawn)
		{
			if (!CanDesignateBreedAnimal(pawn))
				UnDesignateBreedingAnimal(pawn);

			return SaveStorage.DataStore.GetPawnData(pawn).BreedingAnimal;
		}

		public static bool CanDesignateComfort(this Pawn pawn) { return (RJWSettings.WildMode || (pawn.IsPrisonerOfColony || xxx.is_slave(pawn) || (xxx.is_masochist(pawn) && pawn.IsColonist) || pawn.IsDesignatedHero()) && (xxx.can_fuck(pawn) || xxx.can_be_fucked(pawn))); }
		public static bool CanDesignateService(this Pawn pawn) { return (RJWSettings.WildMode || (pawn.IsColonist || pawn.IsPrisonerOfColony || pawn.IsDesignatedHero()) && (xxx.can_fuck(pawn) || xxx.can_be_fucked(pawn))); }
		public static bool CanDesignateBreed(this Pawn pawn)
		{
			if (RJWSettings.bestiality_enabled
				&& xxx.is_human(pawn)
				&& xxx.can_be_fucked(pawn)
				&& (RJWSettings.WildMode || (pawn.IsPrisonerOfColony || xxx.is_slave(pawn) || (xxx.is_zoophile(pawn) && pawn.IsColonist)) || pawn.IsDesignatedHero()))
				return true;

			if (RJWSettings.animal_on_animal_enabled
				&& xxx.is_animal(pawn)
				&& xxx.can_be_fucked(pawn)
				&& pawn.Faction == Faction.OfPlayer)
				return true;

			return false;
		}
		public static bool CanDesignateBreedAnimal(this Pawn pawn)
		{
			//Log.Message("CanDesignateAnimal for " + xxx.get_pawnname(pawn) + " " + SaveStorage.bestiality_enabled);
			//Log.Message("checking animal props " + (pawn.Faction?.IsPlayer.ToString()?? "tynanfag") + xxx.is_animal(pawn) + xxx.can_rape(pawn));
			if ((RJWSettings.bestiality_enabled || RJWSettings.animal_on_animal_enabled)
				&& xxx.is_animal(pawn)
				&& xxx.can_fuck(pawn)
				&& pawn.Faction == Faction.OfPlayer)
					return true;

			return false;

		}
		public static bool CanDesignateMilk(this Pawn pawn) { return false; }//todo
		public static bool CanDesignateHero(this Pawn pawn)
		{
			if ((RJWSettings.RPG_hero_control)
				&& xxx.is_human(pawn)
				&& pawn.IsColonist
				&& pawn.Map.IsPlayerHome)
			{
				foreach (Pawn item in Find.ColonistBar.GetColonistsInOrder())
				//foreach (Pawn item in Find.AnyPlayerHomeMap.mapPawns.AllPawnsSpawned.ToList())
				{
					if (item.IsDesignatedHero())
					{
						return false;
					}
				}
				return true;
			}
			return false;
		}

		public static void DesignateComfort(this Pawn pawn)
		{
			if (pawn.CanDesignateComfort())
			{
				SaveStorage.DataStore.GetPawnData(pawn).Comfort = true;
			}
		}
		public static void DesignateService(this Pawn pawn)
		{
			if (pawn.CanDesignateService())
			{
				SaveStorage.DataStore.GetPawnData(pawn).Service = true;
			}
		}
		public static void DesignateBreeding(this Pawn pawn)
		{
			if (pawn.CanDesignateBreed())
			{
				SaveStorage.DataStore.GetPawnData(pawn).Breeding = true;
			}
		}

		public static void DesignateBreedingAnimal(this Pawn pawn)
		{
			if (pawn.CanDesignateBreedAnimal())
			{
				SaveStorage.DataStore.GetPawnData(pawn).BreedingAnimal = true;
			}
		}

		public static void DesignateMilking(this Pawn pawn)
		{
			if (pawn.CanDesignateMilk())
			{
				SaveStorage.DataStore.GetPawnData(pawn).Milking = true;
			}
		}

		public static void DesignateHero(this Pawn pawn)
		{
			if (pawn.CanDesignateHero())
			{
				SaveStorage.DataStore.GetPawnData(pawn).Hero = true;
				Log.Message(pawn.Name + " is set to hero:" + SaveStorage.DataStore.GetPawnData(pawn).Hero);
			}
		}

		public static void UnDesignateComfort(this Pawn pawn) { SaveStorage.DataStore.GetPawnData(pawn).Comfort = false; }
		public static void UnDesignateService(this Pawn pawn) { SaveStorage.DataStore.GetPawnData(pawn).Service = false; }
		public static void UnDesignateBreeding(this Pawn pawn) { SaveStorage.DataStore.GetPawnData(pawn).Breeding = false; }
		public static void UnDesignateBreedingAnimal(this Pawn pawn) { SaveStorage.DataStore.GetPawnData(pawn).BreedingAnimal = false; }
		public static void UnDesignateMilking(this Pawn pawn) { SaveStorage.DataStore.GetPawnData(pawn).Milking = false; }
		public static void UnDesignateHero(this Pawn pawn) {}

		private static IEnumerable<PawnData> AllDesignationsOn(Map map)
		{
			return SaveStorage.DataStore.PawnData
				.Where(ent => (ent.Value != null && ent.Value.IsValid && map == ent.Value.Pawn.Map))
				.Select(ent => ent.Value)
				.InRandomOrder();
		}
		public static IEnumerable<Pawn> AllComfortDesignations(this Map map)
		{
			return AllDesignationsOn(map)
				.Where(data => (data.Comfort))
				.Select(data => data.Pawn);
		}
		public static IEnumerable<Pawn> AllBreedDesignations(this Map map)
		{
			return AllDesignationsOn(map)
				.Where(data => (data.Breeding))
				.Select(data => data.Pawn);
		}

		public static IEnumerable<Pawn> AllBreedAnimalDesignations(this Map map)
		{
			return AllDesignationsOn(map)
				.Where(data => (data.BreedingAnimal))
				.Select(data => data.Pawn);
		}
		//}
		//public static class PawnDesignations
		//{
		//	public enum Types
		//		{
		//			Comfort,
		//			Service,
		//			Milking,
		//			Breeding
		//		}
		//	public static Dictionary<Types, Func<Pawn, bool>> Checks = new Dictionary<Types, Func<Pawn, bool>>()
		//	{
		//		{ Types.Comfort, CanDesignateComfort}
		//	};
	}
}
