﻿using System.Collections.Generic;
using Verse;
using Harmony;
using UnityEngine;
using System;
using RimWorld;

namespace rjw
{
	[HarmonyPatch(typeof(RimWorld.PawnWoundDrawer))]
	[HarmonyPatch("RenderOverBody")]
	[HarmonyPatch(new Type[] { typeof(Vector3), typeof(Mesh), typeof(Quaternion), typeof(bool) })]
	class patch_semenOverlay
	{

		static void Postfix(RimWorld.PawnWoundDrawer __instance, Vector3 drawLoc, Mesh bodyMesh, Quaternion quat, bool forPortrait)
		{
			Pawn pawn = Traverse.Create(__instance).Field("pawn").GetValue<Pawn>();//get local variable

			if (pawn.RaceProps.Humanlike && RJWSettings.cum_overlays)//for now, only draw humans
			{
				//find bukkake hediff. if it exists, use its draw function
				List<Hediff> hediffs = pawn.health.hediffSet.hediffs;
				if (hediffs.Exists(x => x.def == RJW_HediffDefOf.Hediff_Bukkake))
				{
					Hediff_Bukkake h = hediffs.Find(x => x.def == RJW_HediffDefOf.Hediff_Bukkake) as Hediff_Bukkake;

					quat.ToAngleAxis(out float angle, out Vector3 axis);//angle changes when pawn is e.g. downed

					//adjustments if the pawn is sleeping in a bed:
					bool inBed = false;
					Building_Bed building_Bed = pawn.CurrentBed();
					if (building_Bed != null)
					{
						inBed = !building_Bed.def.building.bed_showSleeperBody;
						AltitudeLayer altLayer = (AltitudeLayer)Mathf.Max((int)building_Bed.def.altitudeLayer, 15);
						Vector3 vector2 = pawn.Position.ToVector3ShiftedWithAltitude(altLayer);
						vector2.y += 0.02734375f+0.01f;//just copied from rimworld code+0.01f
						drawLoc.y = vector2.y;
					}


					h.DrawSemen(drawLoc, quat, forPortrait, angle);
				}
			}

		}
	}

	//adds new gizmo for adding semen for testing
	[HarmonyPatch(typeof(Verse.Pawn))]
	[HarmonyPatch("GetGizmos")]
	class Patch_AddGizmo
	{
		static void Postfix(Verse.Pawn __instance, ref IEnumerable<Gizmo> __result)
		{

			List<Gizmo> NewList = new List<Gizmo>();

			// copy vanilla entries into the new list
			foreach (Gizmo entry in __result)
			{
				NewList.Add(entry);
			}

			if (Prefs.DevMode && RJWSettings.DevMode)
			{
				Command_Action addSemen = new Command_Action();
				addSemen.defaultDesc = "AddSemenHediff";
				addSemen.defaultLabel = "AddSemen";
				addSemen.action = delegate ()
				{
					Pawn pawn = __instance;

					if (!pawn.Dead && pawn.records != null)
					{
						//get all acceptable body parts:
						IEnumerable<BodyPartRecord> filteredParts = SemenHelper.getAvailableBodyParts(pawn);

						//select random part:
						BodyPartRecord randomPart;
						//filteredParts.TryRandomElement<BodyPartRecord>(out randomPart);
						//for testing - choose either genitals or anus:
						if (Rand.Value > 0.5f)
						{
							randomPart = pawn.RaceProps.body.AllParts.Find(x => x.def == xxx.anus.def);
						}
						else
						{
							randomPart = pawn.RaceProps.body.AllParts.Find(x => x.def == xxx.genitals.def);
						}

						if (randomPart != null)
						{
							SemenHelper.cumOn(pawn, randomPart, 0.2f, null, SemenHelper.CUM_NORMAL);
						}

					};
				};

				NewList.Add(addSemen);
			}

			IEnumerable<Gizmo> output = NewList;

			// make caller use the list
			__result = output;

		}
	}
}
