using System.Collections.Generic;
using Harmony;
using RimWorld;
using RimWorld.Planet;
using UnityEngine;
using Verse;

namespace rjw
{
	[HarmonyPatch(typeof(Hediff_Pregnant), "DoBirthSpawn")]
	internal static class PATCH_Hediff_Pregnant_DoBirthSpawn
	{
		/// <summary>
		/// This one overrides vanilla pregnancy hediff behavior.
		/// 0 - try to find suitable father for debug pregnancy
		/// 1st part if character pregnant and rjw pregnancies enabled - creates rjw pregnancy and instantly births it
		/// 2nd part if character pregnant with rjw pregnancy - birth it
		/// 3rd part - debug - create rjw/vanila pregnancy and birth it
		/// </summary>
		/// <param name="mother"></param>
		/// <param name="father"></param>
		/// <returns></returns>
		[HarmonyPrefix]
		private static bool on_begin_DoBirthSpawn(ref Pawn mother, ref Pawn father)
		{
			//--Log.Message("patches_pregnancy::PATCH_Hediff_Pregnant::DoBirthSpawn() called");
			//var mother_name = xxx.get_pawnname(mother);
			//var father_name = xxx.get_pawnname(father);

			if (mother == null)
			{
				Log.Error("Hediff_Pregnant::DoBirthSpawn() - no mother defined -> exit");
				return false;
			}

			if (father == null)
			{
				Log.Warning("Hediff_Pregnant::DoBirthSpawn() - no father defined");
				//birthing with debug has no father
				//make father lover
				if (father == null)
				{
					Pawn target = mother.relations.GetFirstDirectRelationPawn(PawnRelationDefOf.Lover);
					if (target != null)
						father = Genital_Helper.has_penis(target) ? target : null;
				}

				//make father spouse
				if (father == null)
				{
					Pawn target = mother.relations.GetFirstDirectRelationPawn(PawnRelationDefOf.Spouse);
					if (target != null)
						father = Genital_Helper.has_penis(target) ? target : null;
				}

				//make father bonded animal
				//this is for testing only... so far
				if (father == null)
				{
					Pawn target = mother.relations.GetFirstDirectRelationPawn(PawnRelationDefOf.Bond);
					if (target != null)
						father = Genital_Helper.has_penis(target) ? target : null;
				}
			}

			//debug?
			if (mother.gender == Gender.Male)
			{
				Log.Error("Hediff_Pregnant::DoBirthSpawn() - mother is male -> exit");
				return false;
			}

			// get a reference to the hediff we are applying
			//do birth for vanilla pregnancy, if using rjw - add RJW pregnancy and birth it instead
			Hediff_Pregnant self = (Hediff_Pregnant)mother.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("Pregnant"));
			if (self != null)
			{
				Log.Message("patches_pregnancy::PATCH_Hediff_Pregnant::DoBirthSpawn():Vanilla_pregnancy birthing:" + xxx.get_pawnname(mother));
				if (RJWPregnancySettings.animal_pregnancy_enabled && ((xxx.is_animal(father) || father == null) && xxx.is_animal(mother)))
				{
					//RJW Bestial pregnancy animal-animal
					Log.Message(" override as Bestial birthing(animal-animal):" + xxx.get_pawnname(mother));
					Hediff_BestialPregnancy.Create(mother, father);
					Hediff_BestialPregnancy hediff = (Hediff_BestialPregnancy)mother.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("RJW_pregnancy_beast"));
					hediff.GiveBirth();
					if (self != null)
						mother.health.RemoveHediff(self);

					return false;
				}
				else if (RJWPregnancySettings.bestial_pregnancy_enabled && ((xxx.is_animal(father) && xxx.is_human(mother)) || (xxx.is_human(father) && xxx.is_animal(mother))))
				{
					//RJW Bestial pregnancy human-animal
					Log.Message(" override as Bestial birthing(human-animal):" + xxx.get_pawnname(mother));
					Hediff_BestialPregnancy.Create(mother, father);
					Hediff_BestialPregnancy hediff = (Hediff_BestialPregnancy)mother.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("RJW_pregnancy_beast"));
					hediff.GiveBirth();
					if (self != null)
						mother.health.RemoveHediff(self);

					return false;
				}
				else if (RJWPregnancySettings.humanlike_pregnancy_enabled)
				{
					//RJW Humanlike pregnancy
					Log.Message(" override as Humanlike birthing:" + xxx.get_pawnname(mother));
					Hediff_HumanlikePregnancy.Create(mother, father);
					Hediff_HumanlikePregnancy hediff = (Hediff_HumanlikePregnancy)mother.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("RJW_pregnancy"));
					hediff.GiveBirth();
					if (self != null)
						mother.health.RemoveHediff(self);

					return false;
				}
				else
				{
					Log.Warning("Hediff_Pregnant::DoBirthSpawn() - vanilla pregnancy birth");
					//vanilla code
					int num = (mother.RaceProps.litterSizeCurve == null) ? 1 : Mathf.RoundToInt(Rand.ByCurve(mother.RaceProps.litterSizeCurve));
					if (num < 1)
					{
						num = 1;
					}
					PawnGenerationRequest request = new PawnGenerationRequest(mother.kindDef, mother.Faction, PawnGenerationContext.NonPlayer, -1, forceGenerateNewPawn: false, newborn: true);
					Pawn pawn = null;
					for (int i = 0; i < num; i++)
					{
						pawn = PawnGenerator.GeneratePawn(request);
						if (PawnUtility.TrySpawnHatchedOrBornPawn(pawn, mother))
						{
							if (pawn.playerSettings != null && mother.playerSettings != null)
							{
								pawn.playerSettings.AreaRestriction = mother.playerSettings.AreaRestriction;
							}
							if (pawn.RaceProps.IsFlesh)
							{
								pawn.relations.AddDirectRelation(PawnRelationDefOf.Parent, mother);
								if (father != null)
								{
									pawn.relations.AddDirectRelation(PawnRelationDefOf.Parent, father);
								}
							}
						}
						else
						{
							Find.WorldPawns.PassToWorld(pawn, PawnDiscardDecideMode.Discard);
						}
						TaleRecorder.RecordTale(TaleDefOf.GaveBirth, mother, pawn);
					}
					if (mother.Spawned)
					{
						FilthMaker.MakeFilth(mother.Position, mother.Map, ThingDefOf.Filth_AmnioticFluid, mother.LabelIndefinite(), 5);
						if (mother.caller != null)
						{
							mother.caller.DoCall();
						}
						if (pawn.caller != null)
						{
							pawn.caller.DoCall();
						}
					}
					if (self != null)
						mother.health.RemoveHediff(self);

					return false;
				}
			}

			// do birth for RJW pregnancies
			Hediff_HumanlikePregnancy rjwH = (Hediff_HumanlikePregnancy)mother.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("RJW_pregnancy"));
			if (rjwH != null)
			{
				//RJW Bestial pregnancy
				Log.Message("patches_pregnancy::PATCH_Hediff_Pregnant::DoBirthSpawn():RJW_pregnancy birthing:" + xxx.get_pawnname(mother));
				rjwH.GiveBirth();
				if (self == null)
					return false;
			}

			Hediff_BestialPregnancy rjwB = (Hediff_BestialPregnancy)mother.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("RJW_pregnancy_beast"));
			if (rjwB != null)
			{
				//RJW Humanlike pregnancy
				Log.Message("patches_pregnancy::PATCH_Hediff_Pregnant::DoBirthSpawn():RJW_pregnancy_beast birthing:" + xxx.get_pawnname(mother));
				rjwB.GiveBirth();
				if (self == null)
					return false;
			}

			//debug, add RJW pregnancy and birth it
			Log.Message("patches_pregnancy::PATCH_Hediff_Pregnant::DoBirthSpawn():Debug_pregnancy birthing:" + xxx.get_pawnname(mother));
			/*
			if (true)
			{
				//RJW Mech pregnancy
				Log.Message(" override as mech birthing:" + xxx.get_pawnname(mother));
				Hediff_MechanoidPregnancy.Create(mother, father);
				Hediff_MechanoidPregnancy hediff = (Hediff_MechanoidPregnancy)mother.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("RJW_pregnancy_mech"));
				hediff.GiveBirth();
				return false;
			}
			*/

			if (((xxx.is_animal(father) || xxx.is_animal(mother)) && RJWPregnancySettings.bestial_pregnancy_enabled)
				|| (xxx.is_animal(mother) && RJWPregnancySettings.animal_pregnancy_enabled))
			{
				//RJW Bestial pregnancy
				Log.Message(" override as Bestial birthing:" + xxx.get_pawnname(mother));
				Hediff_BestialPregnancy.Create(mother, father);
				Hediff_BestialPregnancy hediff = (Hediff_BestialPregnancy)mother.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("RJW_pregnancy_beast"));
				hediff.GiveBirth();
			}
			else if (RJWPregnancySettings.humanlike_pregnancy_enabled)
			{
				//RJW Humanlike pregnancy
				Log.Message(" override as Humanlike birthing:" + xxx.get_pawnname(mother));
				Hediff_HumanlikePregnancy.Create(mother, father);
				Hediff_HumanlikePregnancy hediff = (Hediff_HumanlikePregnancy)mother.health.hediffSet.GetFirstHediffOfDef(HediffDef.Named("RJW_pregnancy"));
				hediff.GiveBirth();
			}
			else
			{
				Log.Warning("Hediff_Pregnant::DoBirthSpawn() - debug vanilla pregnancy birth");
				//vanilla code
				int num = (mother.RaceProps.litterSizeCurve == null) ? 1 : Mathf.RoundToInt(Rand.ByCurve(mother.RaceProps.litterSizeCurve));
				if (num < 1)
				{
					num = 1;
				}
				PawnGenerationRequest request = new PawnGenerationRequest(mother.kindDef, mother.Faction, PawnGenerationContext.NonPlayer, -1, forceGenerateNewPawn: false, newborn: true);
				Pawn pawn = null;
				for (int i = 0; i < num; i++)
				{
					pawn = PawnGenerator.GeneratePawn(request);
					if (PawnUtility.TrySpawnHatchedOrBornPawn(pawn, mother))
					{
						if (pawn.playerSettings != null && mother.playerSettings != null)
						{
							pawn.playerSettings.AreaRestriction = mother.playerSettings.AreaRestriction;
						}
						if (pawn.RaceProps.IsFlesh)
						{
							pawn.relations.AddDirectRelation(PawnRelationDefOf.Parent, mother);
							if (father != null)
							{
								pawn.relations.AddDirectRelation(PawnRelationDefOf.Parent, father);
							}
						}
					}
					else
					{
						Find.WorldPawns.PassToWorld(pawn, PawnDiscardDecideMode.Discard);
					}
					TaleRecorder.RecordTale(TaleDefOf.GaveBirth, mother, pawn);
				}
				if (mother.Spawned)
				{
					FilthMaker.MakeFilth(mother.Position, mother.Map, ThingDefOf.Filth_AmnioticFluid, mother.LabelIndefinite(), 5);
					if (mother.caller != null)
					{
						mother.caller.DoCall();
					}
					if (pawn.caller != null)
					{
						pawn.caller.DoCall();
					}
				}
				if (self != null)
					mother.health.RemoveHediff(self);
			}
			return false;
		}
	}

	
	[HarmonyPatch(typeof(Hediff_Pregnant), "Tick")]
	class PATCH_Hediff_Pregnant_Tick {
		[HarmonyPrefix]
		static bool on_begin_Tick( Hediff_Pregnant __instance ) {
			if (__instance.pawn.IsHashIntervalTick(1000)) {
				//	//--Log.Message("patches_pregnancy::PATCH_Hediff_Pregnant::Tick( " + __instance.xxx.get_pawnname(pawn) + " ) - gestation_progress = " + __instance.GestationProgress);
				//	if (__instance.Severity < 0.95f) {
				//		__instance.Severity = 0.95f;
				//	}
				if (!Genital_Helper.has_vagina(__instance.pawn))
				{
					__instance.pawn.health.RemoveHediff(__instance);
				}

			}
			return true;
		}
	}
}