﻿using System.Collections.Generic;
using RimWorld;
using RimWorld.Planet;
using Verse;
using System.Text;
using Verse.AI.Group;

namespace rjw
{
	internal class Hediff_InsectEgg : HediffWithComps
	{
		/*public override void Tick()
		{
			base.Tick();
			//--Log.Message("[RJW]Hediff_InsectEgg::Tick() - InsectEgg growing");
		}*/

		protected int bornTick
		{
			get
			{
				return xxx.has_quirk(pawn, "Incubator") ? ((HediffDef_InsectEgg)this.def).bornTick / 2 : ((HediffDef_InsectEgg)this.def).bornTick;
			}
		}

		protected int abortTick
		{
			get
			{
				return ((HediffDef_InsectEgg)this.def).abortTick;
			}
		}

		public float GestationProgress
		{
			get => Severity;
			private set => Severity = value;
		}

		public Pawn father;					//can be parentkind defined in egg
		public Pawn queen;					//can be any pawn
		public bool fertilized = false;
		protected List<Pawn> babies;

		///Contractions duration, effectively additional hediff stage, a dirty hack to make birthing process notable
		//protected const int TicksPerHour = 2500;
		protected int contractions = 0;

		public string parentDef
		{
			get
			{
				return ((HediffDef_InsectEgg)def).parentDef;
			}
		}

		public List<string> parentDefs
		{
			get
			{
				return ((HediffDef_InsectEgg)def).parentDefs;
			}
		}

		public override void PostAdd(DamageInfo? dinfo)
		{
			//--Log.Message("[RJW]Hediff_InsectEgg::PostAdd() - added parentDef:" + parentDef+"");
			base.PostAdd(dinfo);
		}

		public override void Tick()
		{
			this.ageTicks++;
			if (this.pawn.IsHashIntervalTick(1000))
			{
				if (this.ageTicks >= bornTick)
				{
					if (PawnUtility.ShouldSendNotificationAbout(this.pawn))
					{
						string key1 = "RJW_GaveBirthEggTitle";
						string message_title = TranslatorFormattedStringExtensions.Translate(key1, pawn.LabelIndefinite());
						string key2 = "RJW_GaveBirthEggText";
						string message_text = TranslatorFormattedStringExtensions.Translate(key2, pawn.LabelIndefinite());
						//Find.LetterStack.ReceiveLetter(message_title, message_text, LetterDefOf.NeutralEvent, pawn, null);
						Messages.Message(message_text, pawn, MessageTypeDefOf.SituationResolved);
					}
					GiveBirth();
					//someday add dmg to vag?
					//var dam = Rand.RangeInclusive(0, 1);
					//p.TakeDamage(new DamageInfo(DamageDefOf.Burn, dam, 999, -1.0f, null, rec, null));
				}
				else
				{
					//birthing takes an hour
					if (this.ageTicks >= bornTick - 2500 && contractions == 0)
					{
						if (PawnUtility.ShouldSendNotificationAbout(this.pawn))
						{
							string key = "RJW_EggContractions";
							string text = TranslatorFormattedStringExtensions.Translate(key, pawn.LabelIndefinite());
							Messages.Message(text, pawn, MessageTypeDefOf.NeutralEvent);
						}
						contractions++;
						pawn.health.AddHediff(HediffDef.Named("Hediff_Submitting"));
					}
				}

			}
		}

		public override void ExposeData()
		{
			base.ExposeData();
			Scribe_References.Look<Pawn>(ref this.father, "father", false);
			Scribe_References.Look<Pawn>(ref this.queen, "queen", false);
			Scribe_Collections.Look(ref babies, saveDestroyedThings: true, label: "babies", lookMode: LookMode.Deep, ctorArgs: new object[0]);
			Scribe_Values.Look<bool>(ref this.fertilized, "fertilized", false);
		}
		public override void Notify_PawnDied()
		{
			base.Notify_PawnDied();
			GiveBirth();
		}

		protected virtual void GenerateBabies()
		{

		}

		//should someday remake into birth eggs and then within few ticks hatch them
		public void GiveBirth()
		{
			Pawn mother = pawn;
			Pawn baby = null;
			if (fertilized && father != null)
			{
				//Log.Message("[RJW]Hediff_InsectEgg::BirthBaby() - Egg of " + parentDef + " in " + mother.ToString() + " birth!");
				PawnKindDef spawn_kind_def = father.kindDef;
				Faction spawn_faction = null;

				//this is probably fucked up, idk how to filter insects from non insects/spiders etc
				//humanlikes egging
				if (spawn_kind_def.defaultFactionType.humanlikeFaction)
					spawn_faction = queen.Faction;
				//Hive Insects... probably
				else if (spawn_kind_def.defaultFactionType.permanentEnemy)
					if (queen.Faction == Faction.OfInsects || father.Faction == Faction.OfInsects || mother.Faction == Faction.OfInsects)
					{
						spawn_faction = Faction.OfInsects;
						int chance = 5;

						//random chance to make insect neutral/tamable
						if (father.Faction == Faction.OfInsects)
							chance = 5;
						if (father.Faction != Faction.OfInsects)
							chance = 10;
						if (father.Faction == Faction.OfPlayer)
							chance = 25;
						if (queen.Faction == Faction.OfPlayer)
							chance += 25;
						if (queen.Faction == Faction.OfPlayer && xxx.is_human(queen))
							chance += (int)(25 * queen.GetStatValue(StatDefOf.PsychicSensitivity));
						if (Rand.Range(0, 100) <= chance)
							spawn_faction = null;

						//chance tame insect on birth 
						if (spawn_faction == null)
							if (queen.Faction == Faction.OfPlayer && xxx.is_human(queen))
								if (Rand.Range(0, 100) <= (int)(50 * queen.GetStatValue(StatDefOf.PsychicSensitivity)))
									spawn_faction = Faction.OfPlayer;
					}
				//egging animal spawn factionless

				//Log.Message("[RJW]Hediff_InsectEgg::BirthBaby() " + spawn_kind_def + " of " + spawn_faction + " in " + (int)(50 * queen.GetStatValue(StatDefOf.PsychicSensitivity)) + " chance!");
				PawnGenerationRequest request = new PawnGenerationRequest(spawn_kind_def, spawn_faction, PawnGenerationContext.NonPlayer, -1, false, true, false, false, false, false, 0, false, true, false, false, false, false, false, null, null, 0, 0);

				baby = PawnGenerator.GeneratePawn(request);
				if (PawnUtility.TrySpawnHatchedOrBornPawn(baby, mother))
				{
					Genital_Helper.sexualize_pawn(baby);
					if (spawn_faction == Faction.OfInsects)
					{
						//add ai to pawn?
						LordJob_DefendAndExpandHive lordJob = new LordJob_DefendAndExpandHive(true);
						Lord lord = LordMaker.MakeNewLord(baby.Faction, lordJob, baby.Map);
						lord.AddPawn(baby);
						Log.Message("[RJW]Hediff_InsectEgg::BirthBaby() " + baby.GetLord().DebugString());

					}
				}
				else
				{
					Find.WorldPawns.PassToWorld(baby, PawnDiscardDecideMode.Discard);
				}

				// Move the baby in front of the mother, rather than on top
				if (mother.CurrentBed() != null)
				{
					baby.Position = baby.Position + new IntVec3(0, 0, 1).RotatedBy(mother.CurrentBed().Rotation);
				}

				/*
				if (Visible && baby != null)
				{
					string key = "MessageGaveBirth";
					string text = TranslatorFormattedStringExtensions.Translate(key, mother.LabelIndefinite()).CapitalizeFirst();
					Messages.Message(text, baby, MessageTypeDefOf.NeutralEvent);
				}
				*/

				mother.records.AddTo(xxx.CountOfBirthInsect, 1);

				if (mother.records.GetAsInt(xxx.CountOfBirthInsect) > 100)
				{
					if (!xxx.has_quirk(mother, "Incubator"))
					{
						CompRJW.Comp(mother).quirks.AppendWithComma("Incubator");
					}
					if (!xxx.has_quirk(mother, "Impregnation fetish"))
					{
						CompRJW.Comp(mother).quirks.AppendWithComma("Impregnation fetish");
					}
				}
			}
			else
			{
				string key = "EggDead";
				string text = TranslatorFormattedStringExtensions.Translate(key, pawn.LabelIndefinite()).CapitalizeFirst();
				Messages.Message(text, pawn, MessageTypeDefOf.SituationResolved);
			}
			// Post birth
			if (mother.Spawned)
			{
				// Spawn guck
				if (mother.caller != null)
				{
					mother.caller.DoCall();
				}
				if (baby != null)
				{
					if (baby.caller != null)
					{
						baby.caller.DoCall();
					}
				}
				FilthMaker.MakeFilth(mother.Position, mother.Map, ThingDefOf.Filth_AmnioticFluid, mother.LabelIndefinite(), 5);
				int howmuch = xxx.has_quirk(mother, "Incubator") ? Rand.Range(1, 3) * 2 : Rand.Range(1, 3);
				DebugThingPlaceHelper.DebugSpawn(ThingDef.Named("InsectJelly"), mother.InteractionCell, howmuch, false);
			}

			mother.health.RemoveHediff(this);
		}

		public void Fertilize(Pawn pawn)
		{
			if (father == null && ageTicks < abortTick)
			{
				father = pawn;
				fertilized = true;
			}
		}

		public void Implanter(Pawn pawn)
		{
			if (queen == null)
			{
				queen = pawn;
				if (((HediffDef_InsectEgg)this.def).selffertilized)
					Fertilize(pawn);
			}
		}

		public override bool TryMergeWith(Hediff other)
		{
			return false;
		}

		public bool IsParent(string defnam)
		{
			return parentDef == defnam || parentDefs.Contains(defnam);
		}

		public override string DebugString()
		{
			return base.DebugString() + "  Implanter: " + xxx.get_pawnname(queen) + "\n  Age: " + (float)(this.ageTicks/this.bornTick) + "\n  Fertilized:" + (fertilized).ToString() ;
		}
	}
}