using RimWorld;
using Verse;
using Verse.AI;
using System.Collections.Generic;
using System.Linq;

namespace rjw
{
	public class JobGiver_DoFappin : ThinkNode_JobGiver
	{
		public virtual IntVec3 FindFapLocation(Pawn p)
		{
			IntVec3 position = p.Position;
			int bestPosition = -100;
			IntVec3 cell = p.Position;
			int maxDistance = 40;

			FloatRange temperature = p.ComfortableTemperatureRange();
			bool is_somnophile = xxx.has_quirk(p, "Somnophile");
			bool is_exhibitionist = xxx.has_quirk(p, "Exhibitionist");
			List<Pawn> all_pawns = p.Map.mapPawns.AllPawnsSpawned.Where(x 
				=> x.Position.DistanceTo(p.Position) < 100
				&& xxx.is_human(x)
				&& x != p
				).ToList();

			//Log.Message("[RJW] Pawn is " + xxx.get_pawnname(p) + ", current cell is " + cell);

			List<IntVec3> random_cells = new List<IntVec3>();
			for (int loop = 0; loop < 50; ++loop)
			{
				random_cells.Add(position + IntVec3.FromVector3(Vector3Utility.HorizontalVectorFromAngle(Rand.Range(0, 360)) * Rand.RangeInclusive(1, maxDistance)));
			}

			random_cells = random_cells.Where(x 
				=> x.Standable(p.Map) 
				&& x.InAllowedArea(p) 
				&& x.GetDangerFor(p, p.Map) != Danger.Deadly 
				&& !x.ContainsTrap(p.Map) 
				&& !x.ContainsStaticFire(p.Map)
				).Distinct().ToList();

			//Log.Message("[RJW] Found " + random_cells.Count + " valid cells.");

			foreach (IntVec3 random_cell in random_cells)
			{
				int score = 0;
				Room room = random_cell.GetRoom(p.Map);

				bool might_be_seen = all_pawns.Any(x 
					=> GenSight.LineOfSight(x.Position, random_cell, p.Map) 
					&& x.Position.DistanceTo(random_cell) < 50 
					&& x.Awake()
					);

				if (is_exhibitionist)
				{
					if (might_be_seen)
						score += 5;
					else
						score -= 10;
				}
				else
				{
					if (might_be_seen)
						score -= 30;
				}
				if (is_somnophile) // Fap while Watching someone sleep. Not creepy at all!
				{
					if (all_pawns.Any(x 
						=> GenSight.LineOfSight(random_cell, x.Position, p.Map) 
						&& x.Position.DistanceTo(random_cell) < 6 
						&& !x.Awake()
						))
						score += 50;
				}

				if (random_cell.GetTemperature(p.Map) > temperature.min && random_cell.GetTemperature(p.Map) < temperature.max)
					score += 20;
				else
					score -= 20;
				if (random_cell.Roofed(p.Map))
					score += 5;
				if (random_cell.HasEatSurface(p.Map))
					score += 5; // Hide in vegetation.
				if (random_cell.GetDangerFor(p, p.Map) == Danger.Some)
					score -= 25;
				else if (random_cell.GetDangerFor(p, p.Map) == Danger.None)
					score += 5;
				if (random_cell.GetTerrain(p.Map) == TerrainDefOf.WaterShallow ||
					random_cell.GetTerrain(p.Map) == TerrainDefOf.WaterMovingShallow ||
					random_cell.GetTerrain(p.Map) == TerrainDefOf.WaterOceanShallow)
					score -= 20;

				if (random_cell.GetThingList(p.Map).Any(x => x.def.IsWithinCategory(ThingCategoryDefOf.Corpses)) && !xxx.is_necrophiliac(p))
					score -= 20;
				if (random_cell.GetThingList(p.Map).Any(x => x.def.IsWithinCategory(ThingCategoryDefOf.Foods)))
					score -= 10;

				if (room == p.Position.GetRoom(p.MapHeld))
					score -= 10;
				if (room.PsychologicallyOutdoors)
					score += 5;
				if (room.isPrisonCell)
					score += 5;
				if (room.IsHuge)
					score -= 5;
				if (room.ContainedBeds.Any())
					score += 5;
				if (room.IsDoorway)
					score -= 10;
				if (!room.Owners.Any())
					score += 10;
				else if (room.Owners.Contains(p))
					score += 20;
				if (room.Role == RoomRoleDefOf.Bedroom || room.Role == RoomRoleDefOf.PrisonCell)
					score += 10;
				else if (room.Role == RoomRoleDefOf.Barracks || room.Role == RoomRoleDefOf.Laboratory || room.Role == RoomRoleDefOf.RecRoom)
					score += 2;
				else if (room.Role == RoomRoleDefOf.DiningRoom || room.Role == RoomRoleDefOf.Hospital || room.Role == RoomRoleDefOf.PrisonBarracks)
					score -= 5;
				if (room.GetStat(RoomStatDefOf.Cleanliness) < 0.01f)
					score -= 5;
				if (room.GetStat(RoomStatDefOf.GraveVisitingJoyGainFactor) > 0.1f)
					score -= 5;

				if (score <= bestPosition) continue;

				bestPosition = score;
				cell = random_cell;
			}

			return cell;

			//Log.Message("[RJW] Best cell is " + cell);
		}

		protected override Job TryGiveJob(Pawn fapper)
		{
			//--Log.Message("[RJW] JobGiver_DoFappin::TryGiveJob( " + xxx.get_pawnname(fapper) + " ) called");

			if (!xxx.can_be_fucked(fapper) && !xxx.can_fuck(fapper)) return null;

			// Whores only fap if frustrated, unless imprisoned.
			if ((SexUtility.ReadyForLovin(fapper) && (!xxx.is_whore(fapper) || fapper.IsPrisoner )) || xxx.is_frustrated(fapper))
			{
				if (fapper.jobs.curDriver is JobDriver_LayDown)
				{
					Building_Bed bed = ((JobDriver_LayDown)fapper.jobs.curDriver).Bed;
					if (bed != null) return new Job(xxx.fappin, bed);
				}
				else if (xxx.is_frustrated(fapper) || xxx.has_quirk(fapper, "Exhibitionist"))
				{
					return new Job(xxx.quickfap, FindFapLocation(fapper));
				}
			}
			return null;
		}
	}
}